@@ -1,4 +1,5 @@
 HYDROGEN = 1.007825
+PROTON = 1.00727
 #NEUTRON = 1.008701
 # we use the mass of a neutron of c13
 NEUTRON = 1.003355
@@ -14,28 +15,29 @@ ENZYMES = {'trypsin': '[KR]|{P}', 'lysc': '[K]|[X]', 'none': '|',
            'V8': '[ED]|[X]'}
 
 #masses from: http://www.weddslist.com/ms/tables.html#tm4
-#stored as mass, charge
-RESIDUE_MASSES =  {'G': (57.021464, 0),
-                    'A': (71.037114, 0),
-                    'S': (87.032028, 0),
-                    'P': (97.052764, 0),
-                    'V': (99.068414, 0),
-                    'T': (101.047678, 0),
-                    'C': (103.009184, 0),
-                    'I': (113.084064, 0),
-                    'L': (113.084064, 0),
-                    'N': (114.042927, 0),
-                    'D': (115.026943, 0),
-                    'Q': (128.058578, 0),
-                    'K': (128.094963, 1),
-                    'E': (129.042593, 0),
-                    'M': (131.040485, 0),
-                    'H': (137.058912, 1),
-                    'F': (147.068414, 0),
-                    'R': (156.101111, 1),
-                    'Y': (163.063329, 0),
-                    'W': (186.079313, 0)
-                    }
+#stored as mass, charge. Note these are the masses after a water loss
+RESIDUE_MASSES =  {
+  'G': (57.021464, 0),
+  'A': (71.037114, 0),
+  'S': (87.032028, 0),
+  'P': (97.052764, 0),
+  'V': (99.068414, 0),
+  'T': (101.047678, 0),
+  'C': (103.009184, 0),
+  'I': (113.084064, 0),
+  'L': (113.084064, 0),
+  'N': (114.042927, 0),
+  'D': (115.026943, 0),
+  'Q': (128.058578, 0),
+  'K': (128.094963, 1),
+  'E': (129.042593, 0),
+  'M': (131.040485, 0),
+  'H': (137.058912, 1),
+  'F': (147.068414, 0),
+  'R': (156.101111, 1),
+  'Y': (163.063329, 0),
+  'W': (186.079313, 0)
+}
 
 RESIDUE_COMPOSITION = {
     'G': {'C': 2, 'H': 5, 'O': 2, 'N': 1},
