@@ -28,6 +28,7 @@ import six
 if six.PY3:
     xrange = range
 from six import string_types
+import xml.etree.cElementTree as cetree
 try:
     from lxml import etree
 except ImportError:
@@ -61,6 +62,21 @@ lastSplit = re.compile(r'.+[/\\](.+)')
 #         return wrapper(view)
 #     return wrapper
 
+def indent_xml(elem, level=0):
+    i = "\n" + level*"  "
+    if len(elem):
+        if not elem.text or not elem.text.strip():
+            elem.text = i + "  "
+        if not elem.tail or not elem.tail.strip():
+            elem.tail = i
+        for elem in elem:
+            indent_xml(elem, level+1)
+        if not elem.tail or not elem.tail.strip():
+            elem.tail = i
+    else:
+        if level and (not elem.tail or not elem.tail.strip()):
+            elem.tail = i
+
 class GenericProteomicIterator(object):
     """
     Generics to be overridden
@@ -88,7 +104,7 @@ class XMLFileNameMixin(object):
         else:
             file_info = etree.iterparse(self.filename.name, tag=('{http://psi.hupo.org/ms/mzml}sourceFile',))
         self.filetype = None
-        identifiers = [('thermo raw', 'thermo'), ('abi wiff file', 'wiff')]
+        identifiers = [('thermo raw', 'thermo'), ('abi wiff file', 'wiff'), ('agilent masshunter format', 'masshunter')]
         for filenode in file_info:
             filetag = filenode[1]
             file_params = dict([(i.get('name'), i.get('value')) for i in filetag.findall('{0}cvParam'.format('{http://psi.hupo.org/ms/mzml}'))])
@@ -102,7 +118,7 @@ class XMLFileNameMixin(object):
     def _get_scan_from_string(self, value, scan=None):
         if self.filetype == 'thermo':
             try:
-                return dict([i.split('=') for i in value.split(' ')]).get('scan', 'No Title')
+                return dict([i.split('=', 1) for i in value.split(' ') if '=' in i]).get('scan', value)
             except:
                 pass
         if self.filetype == 'wiff':
@@ -113,6 +129,11 @@ class XMLFileNameMixin(object):
                 right = value[left:].find(' ')
                 right = -1 if right == -1 else right+left
                 return value[left:right] if right != -1 else value[left:]
+        if self.filetype == 'masshunter':
+            try:
+                return dict([i.split('=', 1) for i in value.split(' ') if '=' in i]).get('scanId', value)
+            except:
+                pass
         return value
 
 class GuessIterator(templates.GenericIterator):
@@ -148,31 +169,88 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
          I just need a parser right now so the specs might not be exactly correct
 
         The returned items are ScanObjects
-        
+
         """
         super(MZMLIterator, self).__init__(filename)
+        cetree.register_namespace('', 'http://psi.hupo.org/ms/mzml')
         self.scans = {}
         self.store = store
         self.full = full
         self.ms_filter = ms_filter
         self.start = start
-        spectra_tags = ('{http://psi.hupo.org/ms/mzml}spectrum', '{http://psi.hupo.org/ms/mzml}chromatogram')
         self.filename.seek(0)
-        self.spectra = etree.parse(self.filename).iter(tag=('{http://psi.hupo.org/ms/mzml}indexedmzML',)) if self.gzip else etree.iterparse(self.filename, tag=('{http://psi.hupo.org/ms/mzml}indexedmzML',), recover=True)
-        index = next(self.spectra)
-        if index:
-            print('mzML index is missing. Please provide files with an index.')
-            self.ra = dict([(self._get_scan_from_string(i.values()[0]), i.text) for i in index[1].findall('{0}indexList/{0}index/'.format('{http://psi.hupo.org/ms/mzml}'))])
-        else:
-            self.ra = {}
-        self.spectra = etree.parse(self.filename).iter(tag=spectra_tags) if self.gzip else etree.iterparse(self.filename, tag=spectra_tags, recover=True)
+        self.spectra = cetree.parse(self.filename).iter() if self.gzip else cetree.iterparse(self.filename)
+        self.ra = {}
+        duplicate_values = set([])
+        invalid_index = False
+        for event, element in self.spectra:
+            if element.tag.endswith('offset'):
+                self.ra[self._get_scan_from_string(element.get('idRef'))] = element.text
+                if element.text in duplicate_values:
+                    invalid_index = True
+                    break
+                duplicate_values.add(element.text)
+            element.clear()
+        if not self.ra or invalid_index:
+            if invalid_index:
+                print('mzML index contains duplicate values. Please rebuild index. Building a new index in memory.')
+            else:
+                print('mzML index is missing. Please provide files with an index. Building index in memory.')
+            self.ra = self.build_index(self.filename)
+        del self.spectra
+        self.filename.seek(0)
+        self.spectra = cetree.parse(self.filename).iter() if self.gzip else cetree.iterparse(self.filename)
         self.scans = {}
         self.startIter = True
         self.db = None
-            
+
     def __iter__(self):
         return self
 
+    def find_tags(self, text, tag):
+        open = '<'+tag+' '
+        pos = text.find(open)
+        start_pos, end_pos = -1, -1
+        if pos != -1:
+            start_pos = pos
+            end = '</'+tag+'>'
+            pos = text.find(end)
+            if pos != -1:
+                end_pos = pos+len(end)
+        return (start_pos, end_pos, tag)
+
+    def build_index(self, file_obj):
+        file_obj.seek(0)
+        new_index = {}
+        character_count = 0
+        UNCONSUMED = ''
+        while True:
+            i = UNCONSUMED+file_obj.read(2**16)
+            UNCONSUMED  = ''
+            if i == '':
+                break
+            while i:
+                start, end, tag = self.find_tags(i, 'spectrum')
+                if start == -1:
+                    start, end, tag = self.find_tags(i, 'chromatogram')
+                if start == -1:
+                    character_count += len(i)
+                    i = ''
+                    continue
+                start_tag_end = i[start:].find('>\n')
+                if start_tag_end == -1:
+                    UNCONSUMED = i
+                    i = ''
+                    character_count += len(i)
+                    continue
+                tag_header = i[start:start+start_tag_end]+'></'+tag+'>'
+                scanId = self._get_scan_from_string(cetree.fromstring(tag_header).get('id'))
+                new_index[scanId] = character_count+start
+                character_count += start+1
+                i = i[start+1:]
+        file_obj.seek(0)
+        return new_index
+
     def unpack_array(self, array, params, namespace='{http://psi.hupo.org/ms/mzml}'):
         if 'zlib compression' in params:
             import zlib
@@ -180,9 +258,9 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
         else:
             array = base64.b64decode(array)
         if '64-bit float' in params:
-            array = struct.unpack('{}d'.format(len(array)/8), array)
+            array = struct.unpack('{}d'.format(int(len(array)/8)), array)
         else:
-            array = struct.unpack('{}f'.format(len(array)/4), array)
+            array = struct.unpack('{}f'.format(int(len(array)/4)), array)
         return array
 
     def parselxml(self, spectra, full=False, namespace='{http://psi.hupo.org/ms/mzml}'):
@@ -204,6 +282,7 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
                 rt = scan_info.get('scan start time', 0)
                 # rt_length = scan_info.get('ion injection time', 0)
                 scanObj.ms_level = ms_level
+                scanObj.centroid = 'centroid spectrum' in spectra_params
                 scanObj.mass = float(precursor_ion)
                 scanObj.charge = charge
                 title = self._get_scan_from_string(spectra.get('id'))#
@@ -220,7 +299,6 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
                 if self.filetype == 'thermo':
                     target_info = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}precursorList/{0}precursor/{0}isolationWindow/{0}cvParam'.format(namespace))])
                     scanObj.product_ion = target_info.get('isolation window target m/z', 0)
-                spectra.clear()
                 return scanObj
             elif spectra.tag == '{0}chromatogram'.format(namespace):
                 if spectra.get('id') == 'TIC':
@@ -235,21 +313,21 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
                     chromObj.times = time_array
                     chromObj.intensities = intensities
                     self.chromatogram = chromObj
-                    spectra.clear()
                     return None
                 else:
                     scanObj = ScanObject()
                     spectra_params = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}cvParam'.format(namespace))])
                     scan_info = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}scanList/{0}scan/{0}cvParam'.format(namespace))])
                     precursor_info = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}precursor/{0}isolationWindow/{0}cvParam'.format(namespace))])
-                    if self.filetype == 'wiff':
-                        charge = 1
-                        ms_level = int(spectra_params.get('ms level', 2))
-                        product_info = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}product/{0}isolationWindow/{0}cvParam'.format(namespace))])
-                        scanObj.product_ion = product_info.get('isolation window target m/z', 0)
-                    else:
-                        ms_level = int(spectra_params.get('ms level', 0))
-                        charge = precursor_info.get('charge state', 0)
+                    charge = 1
+                    ms_level = int(spectra_params.get('ms level', 2))
+                    product_info = dict([(i.get('name'), i.get('value')) for i in spectra.findall('{0}product/{0}isolationWindow/{0}cvParam'.format(namespace))])
+                    scanObj.product_ion = float(product_info.get('isolation window target m/z', 0))
+                    # if self.filetype == 'wiff':
+                    #     pass
+                    # else:
+                    #     ms_level = int(spectra_params.get('ms level', 0))
+                    #     charge = precursor_info.get('charge state', 0)
                     precursor_ion = precursor_info.get('isolation window target m/z', 0)
                     rt = scan_info.get('scan start time', 0)
                     scanObj.ms_level = ms_level
@@ -263,25 +341,28 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
                         title = int(title)
                     except:
                         pass
-                    if title >= self.start and (not self.ms_filter or ms_level==self.ms_filter) and full:
+                    if (not self.ms_filter or ms_level==self.ms_filter) and full:
                         mzmls, intensities = spectra.findall('{0}binaryDataArrayList/'.format(namespace))
                         mzml_params = dict([(i.get('name'), i.get('value')) for i in mzmls.findall('{0}cvParam'.format(namespace))])
                         mzmls = self.unpack_array(mzmls.find('{0}binary'.format(namespace)).text, mzml_params, namespace=namespace)
                         intensity_params = dict([(i.get('name'), i.get('value')) for i in intensities.findall('{0}cvParam'.format(namespace))])
                         intensities = self.unpack_array(intensities.find('{0}binary'.format(namespace)).text, intensity_params, namespace=namespace)
                         scanObj.scans = list(zip(mzmls, intensities))
-                    spectra.clear()
                     return scanObj
         except:
             import traceback
             sys.stderr.write('Error unpacking spectra {}:\n {}'.format(spectra, traceback.format_exc()))
             return None
-        # elif spect:
-        #     raise StopIteration
+            # elif spect:
+            #     raise StopIteration
 
     def next(self):
         if self.spectra:
             spectra = next(self.spectra)
+            tag = spectra[1].tag
+            while not (tag.endswith('spectrum') or tag.endswith('chromatogram')):
+                spectra = next(self.spectra)
+                tag = spectra[1].tag
         else:
             raise StopIteration
         if self.gzip:
@@ -290,12 +371,97 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
                 self.scans[scan.id] = scan
         else:
             scan = self.parselxml(spectra[1], full=self.full)
+        spectra[1].clear()
         return scan
 
     def getChromatogram(self):
         return self.chromatogram
 
-    def getScan(self, id, peptide=None):
+    def writeScans(self, handle=None, scans=None, namespace='{http://psi.hupo.org/ms/mzml}'):
+        # This function takes the existing mzML, strips all scans, and then replaces them with the desired scans
+        if scans is None:
+            return
+        if not hasattr(scans, '__iter__'):
+            scans = [scans]
+
+        initial_pos = self.filename.tell()
+        self.filename.seek(0)
+        xml_iter = etree.parse(self.filename, events=('start',)).iter() if self.gzip else etree.iterparse(self.filename, events=('start',))
+
+        # this is the indexedmzML header
+        xml_info = xml_iter.next()
+        root = xml_info[1]
+        if handle is None:
+            handle = sys.stdout
+        elif isinstance(handle, six.string_types):
+            handle = open(handle, 'w')
+
+        # go until we get to the spectrumlist because it contains scans
+        spectrumList = root.find('{0}mzML/{0}run/{0}spectrumList'.format(namespace))
+        if spectrumList is None:
+            while xml_info[1].tag != '{}spectrumList'.format(namespace):
+                xml_info = xml_iter.next()
+            spectrumList = xml_info[1]
+        indexList = None
+        indexOffset = None
+        fileChecksum = None
+        for xml_info in xml_iter:
+            if xml_info[1].tag == '{}indexList'.format(namespace):
+                indexList = xml_info[1]
+            elif xml_info[1].tag == '{}indexListOffset'.format(namespace):
+                indexOffset = xml_info[1]
+            elif xml_info[1].tag == '{}fileChecksum'.format(namespace):
+                fileChecksum = xml_info[1]
+        attribs = copy.deepcopy(spectrumList.attrib)
+        spectrumList.clear()
+        attribs['count'] = str(len(scans))
+        for key, value in attribs.items():
+            spectrumList.set(key, value)
+        spectrumList.tail = '\n'
+        if indexList is None:
+            indexList = etree.SubElement(root, 'indexList', {'count': '1'})
+        else:
+            indexList.clear()
+            indexList.set('count', '1')
+
+        indexList.tail = '\n'
+        indent_xml(root)
+        contents = etree.tostring(root, pretty_print=True)
+        scan_offset = contents.find('<spectrumList')+14
+        del contents
+        scan_indices = []
+
+        # add our scans to the xml object
+        offset = self.filename.tell()
+        for scan_index, scan_id in enumerate(scans):
+            scan = self.getScan(str(scan_id), xml=True)
+            scan_indices.append({'offset': str(scan_offset), 'idRef': scan.get('id')})
+            scan.set('index', str(scan_id))
+            scan_offset += len(cetree.tostring(scan))
+            scan.tail = '\n'
+            spectrumList.append(scan)
+
+        self.filename.seek(offset)
+        spectrumIndex = etree.SubElement(indexList, 'index', {'name': 'spectrum'})
+        spectrumIndex.tail = '\n'
+        for scan_index, scan_offset_info in enumerate(scan_indices):
+            offset = etree.SubElement(spectrumIndex, 'offset', {'idRef': scan_offset_info['idRef']})
+            offset.text = scan_offset_info['offset']
+        indent_xml(root)
+        contents = etree.tostring(root, pretty_print=True)
+        if indexOffset is not None:
+            index_offset = contents.find('<indexListOffset')
+            indexOffset.text = str(index_offset)
+        if fileChecksum is not None:
+            import hashlib
+            m = hashlib.sha1()
+            m.update(contents[:contents.find('<fileChecksum')].encode())
+            fileChecksum.text = m.hexdigest()
+        handle.write(contents)
+        self.filename.seek(initial_pos)
+
+
+    def getScan(self, id, peptide=None, xml=False):
         try:
             if self.gzip:
                 #read the whole damn thing in...
@@ -314,20 +480,27 @@ class MZMLIterator(XMLFileNameMixin, templates.GenericIterator, GenericProteomic
                         return spectra
                 self.filename.seek(int(self.ra[str(id)]))
                 entry = self.filename.readline()
-                opening_tag = '<spectrum '
-                closing_tag = '</spectrum>'
-                while entry and opening_tag not in entry:
+                opening_tags = ['<spectrum ', '<chromatogram ']
+                while entry:
+                    matching_tags = list(filter(lambda x: x[0] != -1, ((entry.find(i), i) for i in opening_tags)))
+                    if any(matching_tags):
+                        opening_tag = sorted(matching_tags, key=operator.itemgetter(0))[0][1][1:].strip()
+                        break
                     entry = self.filename.readline()
                 if not entry:
                     return None
                 row = [entry]
+                closing_tag = '</{}>'.format(opening_tag)
                 while entry and closing_tag not in entry:
                     entry = self.filename.readline()
                     row.append(entry)
-                spectra = etree.fromstring(''.join(row))
+                if xml:
+                    spectra = etree.fromstring(''.join(row))
+                    return spectra
+                spectra = cetree.fromstring(''.join(row))
             return self.parselxml(spectra, full=True, namespace='')
-        except IndexError:
-            sys.stderr.write('mzml cannot find %s\n'%id)
+        except:
+            sys.stderr.write('Unable to parse scan {}:\n{}\n'.format(id, traceback.format_exc()))
             return None
 
 class PepXMLIterator(XMLFileNameMixin, GenericProteomicIterator, templates.GenericIterator):
@@ -389,7 +562,7 @@ class PepXMLIterator(XMLFileNameMixin, GenericProteomicIterator, templates.Gener
             pepObj.charge = charge
             mass = float(pep_info.get('precursor_neutral_mass', 0))
             # this is the precursor neutral mass, we need m/z
-            pepObj.mass = (mass+(charge*config.HYDROGEN))/charge
+            pepObj.mass = (mass+(charge*config.PROTON))/charge
             rt = float(pep_info.get('retention_time_sec', 0))
             rt = float(rt/60)+int(rt%60)/100.0
             pepObj.rt = rt
@@ -641,8 +814,8 @@ class XTandemXMLIterator(templates.GenericIterator, GenericProteomicIterator):
         rt = element.attrib.get("rt", 'NA')
         scanObj = PeptideObject()
         scanObj.charge = charge
-        # X!Tandem gives M+H, so we reverese this to get the precursor as it appears on the scan
-        scanObj.mass = (premass+(charge-1)*config.HYDROGEN)/charge
+        # X!Tandem gives M+H, so we reverse this to get the precursor as it appears on the scan
+        scanObj.mass = (premass+(charge-1)*config.PROTON)/charge
         if rt:
             rt_match = self.rt_parse.match(rt)
             if rt_match:
@@ -1213,6 +1386,11 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         self.cur.execute(sql)
         for i in self.cur.fetchall():
             self.tmods[i[0]] = i[1:]
+        self.protein_map = {}
+        sql = 'select pa.ProteinID, pa.Description from proteinannotations pa'
+        self.cur.execute(sql)
+        for i in self.cur.fetchall():
+            self.protein_map[i[0]] = i[1]
         if peptide is not None and not isinstance(peptide, (tuple, set, list)):
             peptide = [peptide]
         self.extra = 'and p.Sequence IN ({})'.format(','.join(['"{}"'.format(i) for i in peptide])) if peptide is not None else ''
@@ -1231,6 +1409,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
             #sql = 'select sp.spectrum,p.ConfidenceLevel,p.ConfidenceLevel,p.Sequence,p.PeptideID,pp.ProteinID,p.SpectrumID from spectra sp left join peptides p on (p.SpectrumID=sp.UniqueSpectrumID) left join peptidesproteins pp on (p.PeptideID=pp.PeptideID) where p.PeptideID IS NOT NULL'
             sql = 'select GROUP_CONCAT(p.ConfidenceLevel),GROUP_CONCAT(p.ConfidenceLevel),GROUP_CONCAT(p.Sequence),GROUP_CONCAT(p.PeptideID), GROUP_CONCAT(pp.ProteinID), p.SpectrumID, sh.Charge, sh.RetentionTime, sh.FirstScan, sh.LastScan, mp.FileID from peptides p join peptidesproteins pp on (p.PeptideID=pp.PeptideID) left join spectrumheaders sh on (sh.SpectrumID=p.SpectrumID) left join masspeaks mp on (sh.MassPeakID=mp.MassPeakID) where p.PeptideID IS NOT NULL and p.ConfidenceLevel >= {} {} GROUP BY p.SpectrumID'.format(clvl, self.extra)
             self.cur.execute(sql)
+        self.base_sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID, pp.ProteinID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) left join peptidesproteins pp on (p.PeptideID=pp.PeptideID) "
         self.index = 0
         self.master_scans = {}
 
@@ -1238,6 +1417,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         labels = {}
         from six.moves import html_parser as HTMLParser
         html_parser = HTMLParser.HTMLParser()
+        silac = None
         if self.version == 1:
             sql = 'select ParameterValue from processingnodeparameters where ParameterName == "QuantificationMethod"'
             self.cur.execute(sql)
@@ -1249,6 +1429,8 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
                     silac = etree.fromstring(str(xml).encode('utf-16'))
         elif self.version == 2:
             silac = etree.fromstring([i for i in self.root.iterdescendants('QuantitationMethod')][0].text.encode('utf-16'))
+        if silac is None:
+            return labels
         for method in silac.findall('*MethodPart'):
             if self.version == 1 and method.getparent().get('name') != 'QuanChannels':
                 continue
@@ -1260,8 +1442,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
                         masses[float(mod_info.get('DeltaMass'))].add(aa)
                     except KeyError:
                         masses[float(mod_info.get('DeltaMass'))] = set([aa])
-            if masses:
-                labels[method_label] = masses
+            labels[method_label] = masses
         return labels
 
     def loadChromatogram(self):
@@ -1302,7 +1483,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         """
         get a random scan
         """
-        sql = "select sp.Spectrum, p.Sequence, p.PeptideID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) where sh.SpectrumID = %d and p.Sequence = '%s'"%(int(specId),peptide)
+        sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) where sh.SpectrumID = %d and p.Sequence = '%s'"%(int(specId),peptide)
         self.cur.execute(sql)
         i = self.cur.fetchone()
         if not i:
@@ -1316,14 +1497,14 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         get a random scan
         """
         if fdr:
-            sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) WHERE p.ConfidenceLevel >= {} and p.SearchEngineRank <= {} {}".format(self.clvl, self.srank, self.extra)
+            sql = self.base_sql+"WHERE p.ConfidenceLevel >= {} and p.SearchEngineRank <= {} {}".format(self.clvl, self.srank, self.extra)
             try:
                 self.cur.execute(sql)
             except sqlite3.OperationalError:
-                sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) WHERE p.ConfidenceLevel >= {} {}".format(self.clvl, self.extra)
+                sql = self.base_sql+"WHERE p.ConfidenceLevel >= {} {}".format(self.clvl, self.extra)
                 self.cur.execute(sql)
         else:
-            sql = "select sp.Spectrum, p.Sequence, p.PeptideID, p.SpectrumID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID)"
+            sql = self.base_sql
             self.cur.execute(sql)
         while True:
             # results = self.cur.fetchmany(1000)
@@ -1381,6 +1562,8 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
                         finfo = row.split('"')
                         charge = finfo[3]
                         smass = finfo[17]
+                        if not float(smass):
+                            smass = (float(finfo[5])+(int(charge)-1)*config.PROTON)/float(charge)
                         scanObj.charge = charge
                         scanObj.mass = float(smass)
                         stage=2
@@ -1440,7 +1623,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
             scanObj.peptide = sequence
             scanObj.rank = searchRank
             scanObj.confidence = confidence
-            scanObj.acc = i[4]
+            scanObj.acc = ';'.join(self.protein_map.get(j, j) for j in i[4].split(','))
             scanObj.charge = i[6]
             scanObj.rt = i[7]
             fName = self.sFileMap[i[10]]
@@ -1461,6 +1644,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         scanObj = PeptideObject()
         peptide = str(i[1])
         pid=i[2]
+        scanObj.acc = self.protein_map.get(i[4], i[4])
         if pid is None:
             return None
         if modifications:
@@ -1513,7 +1697,7 @@ class ThermoMSFIterator(templates.GenericIterator, GenericProteomicIterator):
         return self.index*100/self.nrows
 
     def getPeptide(self, peptide=None):
-        sql = "select sp.Spectrum, p.Sequence, p.PeptideID from spectrumheaders sh left join spectra sp on (sp.UniqueSpectrumID=sh.UniqueSpectrumID) left join peptides p on (sh.SpectrumID=p.SpectrumID) where p.Sequence = '%s'"%(peptide)
+        sql = self.base_sql+"where p.Sequence = '%s'"%(peptide)
         self.cur.execute(sql)
         for i in self.cur.fetchall():
             yield self.parseFullScan(i)
