@@ -37,6 +37,8 @@ class ScanObject(object):
         self._rawId = None
         self.parent = None
 
+        self.centroid = False
+
         #SRM/MRM parameters
         self.product_ion = 0
 
@@ -77,6 +79,7 @@ class PeptideObject(ScanObject):
         self.mods = set([])
         self.peptide = ""
         self.hit = 0
+        self.acc = ''# accession information
 
     def addModification(self, aa,position, modMass, modType):
         """
@@ -129,5 +132,5 @@ class PeptideObject(ScanObject):
         aa_info = [config.RESIDUE_MASSES[i.upper()] for i in self.peptide if i in config.RESIDUE_MASSES]
         peptide_mass = sum([i[0] for i in aa_info])+config.MODIFICATION_MASSES['h'][0]+config.MODIFICATION_MASSES['oh'][0]
         modifications = sum([float(i[2]) for i in self.mods])
-        mass = peptide_mass+modifications+float(self.charge)*config.HYDROGEN
+        mass = peptide_mass+modifications+float(self.charge)*config.PROTON
         return mass/float(self.charge)
\ No newline at end of file
